@@ -98,6 +98,7 @@ class ComputationGraph:
         needed for getting reproducible/deterministic node name and
         graphviz graphs. This is especially important for output tests
         '''
+        self.context_tracker = {'current_context': [], 'current_depth': 0}
         self.running_node_id: int = 0
         self.running_subgraph_id: int = 0
         self.id_dict: dict[str, int] = {}
@@ -165,7 +166,7 @@ class ComputationGraph:
 
             # if it is container module, move directly to outputs
             if self.hide_module_functions and k.is_container:
-                for g in k.end_nodes:
+                for g in k.output_nodes:
                     new_kwargs = updated_dict(new_kwargs, 'cur_node', g)
                     self.traverse_graph(action_fn, **new_kwargs)
                 return
@@ -232,24 +233,24 @@ class ComputationGraph:
         tail_node = self.get_tail_node(cur_node)
         is_main_node_visible = self.is_node_visible(cur_node.main_node)
         is_tail_node_visible = self.is_node_visible(tail_node)
-        if not cur_node.is_main_output():
-            for output_node in cur_node.outputs:
-                is_output_visible = self.is_node_visible(output_node)
+        if not cur_node.is_leaf():
+            for children_node in cur_node.children:
+                is_output_visible = self.is_node_visible(children_node)
                 if is_output_visible:
                     if is_main_node_visible:
-                        self.edge_list.append((cur_node, output_node))
+                        self.edge_list.append((cur_node, children_node))
                     elif is_tail_node_visible:
-                        self.edge_list.append((tail_node, output_node))
+                        self.edge_list.append((tail_node, children_node))
 
         # {tail -> cur_node} part
         # # output node
         # visible tensor and non-input tensor nodes
-        if is_cur_visible and not cur_node.is_main_input():
-            assert not isinstance(tail_node, TensorNode) or tail_node.is_main_input()
+        if is_cur_visible and not cur_node.is_root():
+            assert not isinstance(tail_node, TensorNode) or tail_node.is_root()
             self.edge_list.append((tail_node, cur_node))
 
     def rollify(
-        self, cur_node: COMPUTATION_NODES
+        self, cur_node: ModuleNode | FunctionNode
     ):
         '''Rolls computational graph by identifying recursively used
         Modules. This is done by giving the same id for nodes that are
@@ -257,9 +258,9 @@ class ComputationGraph:
         This becomes complex when there are stateless and torch.functions.
         For more details see docs'''
 
-        head_node = next(iter(cur_node.end_nodes))
-        if not head_node.is_main_output() and self.hide_inner_tensors:
-            head_node = next(iter(head_node.outputs))
+        head_node = next(iter(cur_node.output_nodes))
+        if not head_node.is_leaf() and self.hide_inner_tensors:
+            head_node = next(iter(head_node.children))
 
         # identify recursively used modules
         # with the same node id
@@ -286,12 +287,12 @@ class ComputationGraph:
             if compute_node.main_node.depth < 0 or compute_node.is_aux:
                 return False
 
-            is_main_output_or_input = (
-                (compute_node.is_main_input() or compute_node.is_main_output())
+            is_main_input_or_output = (
+                (compute_node.is_root() or compute_node.is_leaf())
                 and compute_node.depth == 0
             )
             is_visible = (
-                not self.hide_inner_tensors or is_main_output_or_input
+                not self.hide_inner_tensors or is_main_input_or_output
             )
 
             return is_visible
@@ -305,27 +306,27 @@ class ComputationGraph:
         tensor_node = _tensor_node.main_node if _tensor_node.is_aux else _tensor_node
 
         # non-output nodes eminating from input node
-        if tensor_node.is_main_input():
+        if tensor_node.is_root():
             return tensor_node
 
-        current_input_h = tensor_node.input_hierarchy
+        current_parent_h = tensor_node.parent_hierarchy
 
-        sorted_depth = sorted(depth for depth in current_input_h)
-        tail_node = next(iter(tensor_node.inputs))
+        sorted_depth = sorted(depth for depth in current_parent_h)
+        tail_node = next(iter(tensor_node.parents))
         depth = 0
         for depth in sorted_depth:
-            tail_node = current_input_h[depth]
+            tail_node = current_parent_h[depth]
             if depth >= self.depth:
                 break
 
         module_depth = depth-1
         # if returned by container module and hide_module_functions
         if (
-            isinstance(current_input_h[depth], FunctionNode) and
-            module_depth in tensor_node.input_hierarchy and self.hide_module_functions
+            isinstance(current_parent_h[depth], FunctionNode) and
+            module_depth in tensor_node.parent_hierarchy and self.hide_module_functions
         ):
-            if current_input_h[module_depth].is_container:
-                return current_input_h[module_depth]
+            if current_parent_h[module_depth].is_container:
+                return current_parent_h[module_depth]
 
         # Even though this is recursive, not harmful for complexity
         # The reason: the (time) complexity ~ O(L^2) where L
@@ -334,12 +335,12 @@ class ComputationGraph:
         # infinitely big network with infinitely big continuou pass of unchanged
         # tensor. This recursion is necessary e.g. for LDC model
         if tail_node.name == 'empty-pass':
-            empty_pass_input = next(iter((tail_node.inputs)))
-            assert isinstance(empty_pass_input, TensorNode), (
-                f'{empty_pass_input} is input of {tail_node}'
+            empty_pass_parent = next(iter((tail_node.parents)))
+            assert isinstance(empty_pass_parent, TensorNode), (
+                f'{empty_pass_parent} is input of {tail_node}'
                 f'and must a be TensorNode'
             )
-            return self.get_tail_node(empty_pass_input)
+            return self.get_tail_node(empty_pass_parent)
         return tail_node
 
     def add_edge(
@@ -424,12 +425,12 @@ class ComputationGraph:
     def check_node(self, node):
         assert node.node_id != 'null', f'wrong id {node} {type(node)}'
         assert '-' not in node.node_id, 'No repetition of node recording'
-        assert not node.is_main_output() or not node.is_main_input(), (
+        assert not node.is_leaf() or not node.is_root(), (
             f'isolated node! {node}'
         )
         assert node.depth <= self.depth, f"exceeds depth limit, {node}"
         assert (
-            sum(1 for _ in node.inputs) in [0, 1] or not isinstance(node, TensorNode)
+            sum(1 for _ in node.parents) in [0, 1] or not isinstance(node, TensorNode)
         ), (
             f'tensor must have single input node {node}'
         )
